@@ -124,7 +124,7 @@
 ##    qic_lin = Quasi information criterion (QIC)
 ################################################################################
 
-log_lin_estimnarpq <- function(y, W, p, Z = NULL) {
+log_lin_estimnarpq <- function(y, W, p, Z = NULL, uncons = FALSE) {
 
   if ( min(W) < 0 ) {
     stop('The adjacency matrix W contains negative values.')
@@ -150,27 +150,33 @@ log_lin_estimnarpq <- function(y, W, p, Z = NULL) {
   x0 <- solve(XX, Xy)
 
   m <- length(x0)
-  # Inequality constraints (parameters searched in the stationary region)
-  # b are the parameters to be constrained
-  constr <- function(b, N, TT, y, W, wy, p, Z) {
-    con <- sum( abs( b[2:(2 * p + 1)] ) ) - 1
-    return(con)
-  }
-
-  # Jacobian of constraints
-  # b are the parameters to be constrained
-  j_constr <- function(b, N, TT, y, W, wy, p, Z) {
-    j_con <- b/abs(b)
-    j_con[1] <- 0
-    return(j_con)
-  }
-
   # algorithm and and relative tolerance
   opts <- list("algorithm" = "NLOPT_LD_SLSQP", "xtol_rel" = 1e-8)
 
-  s_qmle <- nloptr::nloptr(x0 = x0, eval_f = .logl_log_linpq, eval_grad_f = .scor_logpq,
-                    eval_g_ineq = constr, eval_jac_g_ineq = j_constr,
-                    opts = opts, N = N, TT = TT, y = y, W = W, wy = wy, p = p, Z = Z)
+  if( uncons ) {
+    s_qmle <- nloptr::nloptr(x0 = x0, eval_f = .logl_log_linpq, eval_grad_f = .scor_logpq,
+                      opts = opts, N = N, TT = TT, y = y, W = W, wy = wy, p = p, Z = Z)
+  } else {
+
+    # Inequality constraints (parameters searched in the stationary region)
+    # b are the parameters to be constrained
+    constr <- function(b, N, TT, y, W, wy, p, Z) {
+      con <- sum( abs( b[2:(2 * p + 1)] ) ) - 1
+      return(con)
+    }
+
+    # Jacobian of constraints
+    # b are the parameters to be constrained
+    j_constr <- function(b, N, TT, y, W, wy, p, Z) {
+      j_con <- b/abs(b)
+      j_con[1] <- 0
+      return(j_con)
+    }
+
+    s_qmle <- nloptr::nloptr(x0 = x0, eval_f = .logl_log_linpq, eval_grad_f = .scor_logpq,
+                      eval_g_ineq = constr, eval_jac_g_ineq = j_constr, opts = opts,
+                      N = N, TT = TT, y = y, W = W, wy = wy, p = p, Z = Z)
+  }
 
   coeflin <- s_qmle$solution
 
@@ -184,14 +190,27 @@ log_lin_estimnarpq <- function(y, W, p, Z = NULL) {
   SE_lins <- sqrt( diag(V_lins) )
 
   tlin <- coeflin/SE_lins
+  pval <- 2 * pnorm(abs(tlin), lower.tail = FALSE)
 
   loglik <-  - s_qmle$objective
   aic_lins <- 2 * m + 2 * s_qmle$objective
   bic_lins <- log(TT) * m + 2 * s_qmle$objective
   qic_lins <- 2 * sum(H_lins * V_lins ) + 2 * s_qmle$objective
 
-  list( coeflog = coeflin, selog = SE_lins, tlog = tlin, score = S_lins,
-        loglik = loglik, aic_log = aic_lins, bic_log = bic_lins, qic_log = qic_lins)
+  coeflog <- cbind(coeflin, SE_lins, tlin, pval)
+  colnames(coeflog) <- c("Estimate", "Std. Error", "Z-stat", "p-value")
+  if ( !is.null( dim(Z) ) ) {
+    rownames(coeflog) <- rownames(S_lins) <- c( "beta0", paste("beta1", 1:p, sep =""), paste("beta2", 1:p, sep =""),
+                                                paste("delta", 1:dim(Z)[2], sep ="") )
+  } else  rownames(coeflog) <- rownames(S_lins) <- c( "beta0", paste("beta1", 1:p, sep =""), paste("beta2", 1:p, sep ="") )
+  ic <- c(aic_lins, bic_lins, qic_lins)
+  names(ic) <- c("AIC", "BIC", "QIC")
+
+  if ( any( abs(S_lins) > 1e-3 ) )  {
+    warning( paste("Optimization failed in the stationary region. Please try estimation without stationarity constraints.") )
+  }
+
+  list( coeflog = coeflog, score = S_lins, loglik = loglik, ic = ic )
 }
 
 
